@@ -238,8 +238,7 @@ def link_model(
 
     """
 
-    if aliases is None:
-        aliases = ["latest"]
+    aliases = wandb.util._resolve_aliases(aliases)
 
     if wandb.run:
         run = wandb.run
