@@ -570,6 +570,7 @@ def sync(
             view=view,
             verbose=verbose,
             sync_tensorboard=_sync_tensorboard,
+            log_path=_wandb_log_path,
         )
         for p in _path:
             sm.add(p)
@@ -976,6 +977,13 @@ def sweep(
     "or a git uri pointing to a remote repository, or path to a local directory.",
 )
 @click.argument("uri", nargs=1, required=False)
+@click.option(
+    "--job",
+    "-j",
+    metavar="<str>",
+    default=None,
+    help="Name of the job to launch. If passed in, launch does not require a uri.",
+)
 @click.option(
     "--entry-point",
     "-E",
@@ -1084,6 +1092,7 @@ def sweep(
 @display_error
 def launch(
     uri,
+    job,
     entry_point,
     git_version,
     args_list,
@@ -1158,20 +1167,13 @@ def launch(
     elif resource is None:
         resource = "local-container"
 
-    if (
-        uri is None
-        and docker_image is None
-        and config.get("uri") is not None
-        and config.get("docker", {}).get("docker_image") is None
-    ):
-        raise LaunchError("Must pass a URI or a docker image to launch.")
-
     if queue is None:
         # direct launch
         try:
             wandb_launch.run(
-                uri,
                 api,
+                uri,
+                job,
                 entry_point,
                 git_version,
                 project=project,
@@ -1195,6 +1197,7 @@ def launch(
         _launch_add(
             api,
             uri,
+            job,
             config,
             project,
             entity,
@@ -1641,14 +1644,12 @@ def start(ctx, port, env, daemon, upgrade, edge):
     if daemon:
         if code != 0:
             wandb.termerror(
-                "Failed to launch the W&B local container, see the above error."
+                "Failed to launch the W&B server container, see the above error."
             )
             exit(1)
         else:
-            wandb.termlog("W&B local started at http://localhost:%s \U0001F680" % port)
-            wandb.termlog(
-                "You can stop the server by running `docker stop wandb-local`"
-            )
+            wandb.termlog("W&B server started at http://localhost:%s \U0001F680" % port)
+            wandb.termlog("You can stop the server by running `wandb server stop`")
             if not api.api_key:
                 # Let the server start before potentially launching a browser
                 time.sleep(2)
