@@ -1,7 +1,9 @@
 import logging
-import click
+from typing import Any
 import sys
 
+import click
+
 
 LOG_STRING = click.style("wandb", fg="blue", bold=True)
 LOG_STRING_NOCOLOR = "wandb"
@@ -18,14 +20,16 @@ _logger = None
 
 def termsetup(settings, logger):
     global _silent, _show_info, _show_warnings, _show_errors, _logger
-    _silent = settings._silent
-    _show_info = settings._show_info
-    _show_warnings = settings._show_warnings
-    _show_errors = settings._show_errors
+    _silent = settings.silent
+    _show_info = settings.show_info
+    _show_warnings = settings.show_warnings
+    _show_errors = settings.show_errors
     _logger = logger
 
 
-def termlog(string="", newline=True, repeat=True, prefix=True):
+def termlog(
+    string: str = "", newline: bool = True, repeat: bool = True, prefix: bool = True
+) -> None:
     """Log to standard error with formatting.
 
     Arguments:
@@ -42,25 +46,25 @@ def termlog(string="", newline=True, repeat=True, prefix=True):
     )
 
 
-def termwarn(string, **kwargs):
-    string = "\n".join(["{} {}".format(WARN_STRING, s) for s in string.split("\n")])
+def termwarn(string: str, **kwargs: Any) -> None:
+    string = "\n".join([f"{WARN_STRING} {s}" for s in string.split("\n")])
     _log(
         string=string,
         newline=True,
         silent=not _show_warnings,
         level=logging.WARNING,
-        **kwargs
+        **kwargs,
     )
 
 
-def termerror(string, **kwargs):
-    string = "\n".join(["{} {}".format(ERROR_STRING, s) for s in string.split("\n")])
+def termerror(string: str, **kwargs: Any) -> None:
+    string = "\n".join([f"{ERROR_STRING} {s}" for s in string.split("\n")])
     _log(
         string=string,
         newline=True,
         silent=not _show_errors,
         level=logging.ERROR,
-        **kwargs
+        **kwargs,
     )
 
 
@@ -71,9 +75,7 @@ def _log(
     silent = silent or _silent
     if string:
         if prefix:
-            line = "\n".join(
-                ["{}: {}".format(LOG_STRING, s) for s in string.split("\n")]
-            )
+            line = "\n".join([f"{LOG_STRING}: {s}" for s in string.split("\n")])
         else:
             line = string
     else:
