@@ -2,7 +2,7 @@
 Internal utility for converting arguments from a launch spec or call to wandb launch
 into a runnable wandb launch script
 """
-
+import binascii
 import enum
 import json
 import logging
@@ -13,8 +13,9 @@ from typing import Any, Dict, List, Optional
 
 import wandb
 from wandb.apis.internal import Api
+from wandb.apis.public import Artifact as PublicArtifact
 import wandb.docker as docker
-from wandb.errors import Error as ExecutionError, LaunchError
+from wandb.errors import CommError, LaunchError
 from wandb.sdk.lib.runid import generate_id
 
 from . import utils
@@ -27,6 +28,7 @@ DEFAULT_LAUNCH_METADATA_PATH = "launch_metadata.json"
 # need to make user root for sagemaker, so users have access to /opt/ml directories
 # that let users create artifacts and access input data
 RESOURCE_UID_MAP = {"local": 1000, "sagemaker": 0}
+IMAGE_TAG_MAX_LENGTH = 32
 
 
 class LaunchSource(enum.IntEnum):
@@ -34,6 +36,7 @@ class LaunchSource(enum.IntEnum):
     GIT: int = 2
     LOCAL: int = 3
     DOCKER: int = 4
+    JOB: int = 5
 
 
 class LaunchProject:
@@ -42,6 +45,7 @@ class LaunchProject:
     def __init__(
         self,
         uri: Optional[str],
+        job: Optional[str],
         api: Api,
         launch_spec: Dict[str, Any],
         target_entity: str,
@@ -59,11 +63,15 @@ class LaunchProject:
             uri = api.settings("base_url") + uri
             _logger.info(f"Updating uri with base uri: {uri}")
         self.uri = uri
+        self.job = job
+        self._job_artifact: Optional[PublicArtifact] = None
         self.api = api
         self.launch_spec = launch_spec
         self.target_entity = target_entity
         self.target_project = target_project.lower()
         self.name = name  # TODO: replace with run_id
+        self.resource = resource
+        self.resource_args = resource_args
         self.build_image: bool = docker_config.get("build_image", False)
         self.python_version: Optional[str] = docker_config.get("python_version")
         self.cuda_version: Optional[str] = docker_config.get("cuda_version")
@@ -80,37 +88,38 @@ class LaunchProject:
         self.override_config: Dict[str, Any] = overrides.get("run_config", {})
         self.override_artifacts: Dict[str, Any] = overrides.get("artifacts", {})
         self.override_entrypoint: Optional[EntryPoint] = None
-        self.resource = resource
-        self.resource_args = resource_args
         self.deps_type: Optional[str] = None
         self.cuda = cuda
         self._runtime: Optional[str] = None
         self.run_id = run_id or generate_id()
+        self._image_tag: str = self._initialize_image_job_tag() or self.run_id
         self._entry_points: Dict[
             str, EntryPoint
         ] = {}  # todo: keep multiple entrypoint support?
-        if overrides.get("entry_point") is not None:
+
+        if overrides.get("entry_point"):
             _logger.info("Adding override entry point")
             self.override_entrypoint = self.add_entry_point(
                 overrides.get("entry_point")  # type: ignore
             )
-        if self.uri is None:
-            if self.docker_image is None:
-                raise LaunchError("Run requires a URI or a docker image")
+        if self.docker_image is not None:
             self.source = LaunchSource.DOCKER
             self.project_dir = None
-        elif utils._is_wandb_uri(self.uri):
+        elif self.job is not None:
+            self.source = LaunchSource.JOB
+            self.project_dir = tempfile.mkdtemp()
+        elif self.uri is not None and utils._is_wandb_uri(self.uri):
             _logger.info(f"URI {self.uri} indicates a wandb uri")
             self.source = LaunchSource.WANDB
             self.project_dir = tempfile.mkdtemp()
-        elif utils._is_git_uri(self.uri):
+        elif self.uri is not None and utils._is_git_uri(self.uri):
             _logger.info(f"URI {self.uri} indicates a git uri")
             self.source = LaunchSource.GIT
             self.project_dir = tempfile.mkdtemp()
         else:
             _logger.info(f"URI {self.uri} indicates a local uri")
             # assume local
-            if not os.path.exists(self.uri):
+            if self.uri is not None and not os.path.exists(self.uri):
                 raise LaunchError(
                     "Assumed URI supplied is a local path but path is not valid"
                 )
@@ -139,10 +148,43 @@ class LaunchProject:
 
     @property
     def image_name(self) -> str:
-        """Returns {PROJECT}_launch the ultimate version will
-        be tagged with a sha of the git repo"""
-        # TODO: this should likely be source_project when we have it...
-        return f"{self.target_project}_launch"
+        if self.docker_image is not None:
+            return self.docker_image
+        elif self.uri is not None:
+            cleaned_uri = self.uri.replace("https://", "/")
+            first_sep = cleaned_uri.find("/")
+            shortened_uri = cleaned_uri[first_sep:]
+            return wandb.util.make_docker_image_name_safe(shortened_uri)
+        else:
+            # this will always pass since one of these 3 is required
+            assert self.job is not None
+            return wandb.util.make_docker_image_name_safe(self.job.split(":")[0])
+
+    def _initialize_image_job_tag(self) -> Optional[str]:
+        if self.job is not None:
+            _, alias = self.job.split(":")
+            self._image_tag = alias
+            return alias
+        return None
+
+    @property
+    def image_uri(self) -> str:
+        if self.docker_image:
+            return self.docker_image
+        return f"{self.image_name}:{self.image_tag}"
+
+    @property
+    def image_tag(self) -> str:
+        return self._image_tag[:IMAGE_TAG_MAX_LENGTH]
+
+    @property
+    def docker_image(self) -> Optional[str]:
+        return self._docker_image
+
+    @docker_image.setter
+    def docker_image(self, value: str) -> None:
+        self._docker_image = value
+        self._ensure_not_docker_image_and_local_process()
 
     def clear_parameter_run_config_collisions(self) -> None:
         """Clear values from the override run config values if a matching key exists in the override arguments."""
@@ -172,10 +214,26 @@ class LaunchProject:
         self._entry_points[entry_point] = new_entrypoint
         return new_entrypoint
 
+    def _ensure_not_docker_image_and_local_process(self) -> None:
+        if self.docker_image is not None and self.resource == "local-process":
+            raise LaunchError(
+                "Cannot specify docker image with local-process resource runner"
+            )
+
+    def _fetch_job(self) -> None:
+        public_api = wandb.apis.public.Api()
+        job_dir = tempfile.mkdtemp()
+        try:
+            job = public_api.job(self.job, path=job_dir)
+        except CommError:
+            raise LaunchError(f"Job {self.job} not found")
+        job.configure_launch_project(self)
+        self._job_artifact = job._job_artifact
+
     def _fetch_project_local(self, internal_api: Api) -> None:
         """Fetch a project (either wandb run or git repo) into a local directory, returning the path to the local project directory."""
         # these asserts are all guaranteed to pass, but are required by mypy
-        assert self.source != LaunchSource.LOCAL
+        assert self.source != LaunchSource.LOCAL and self.source != LaunchSource.JOB
         assert isinstance(self.uri, str)
         assert self.project_dir is not None
         _logger.info("Fetching project locally...")
@@ -210,6 +268,8 @@ class LaunchProject:
                             self.cuda_version, original_cuda_version, self.cuda_version
                         )
                     )
+            # Specify the python runtime for jupyter2docker
+            self.python_version = run_info.get("python", "3")
 
             downloaded_code_artifact = utils.check_and_download_code_artifacts(
                 source_entity,
@@ -218,12 +278,13 @@ class LaunchProject:
                 internal_api,
                 self.project_dir,
             )
-
             if downloaded_code_artifact:
-                self.build_image = True
-            elif not downloaded_code_artifact:
+                self._image_tag = binascii.hexlify(
+                    downloaded_code_artifact.digest.encode()
+                ).decode()
+            else:
                 if not run_info["git"]:
-                    raise ExecutionError(
+                    raise LaunchError(
                         "Reproducing a run requires either an associated git repo or a code artifact logged with `run.log_code()`"
                     )
                 utils._fetch_git_repo(
@@ -234,9 +295,13 @@ class LaunchProject:
                 patch = utils.fetch_project_diff(
                     source_entity, source_project, source_run_name, internal_api
                 )
-
+                tag_string = run_info["git"]["remote"] + run_info["git"]["commit"]
                 if patch:
                     utils.apply_patch(patch, self.project_dir)
+                    tag_string += patch
+
+                self._image_tag = binascii.hexlify(tag_string.encode()).decode()
+
                 # For cases where the entry point wasn't checked into git
                 if not os.path.exists(os.path.join(self.project_dir, program_name)):
                     downloaded_entrypoint = utils.download_entry_point(
@@ -252,9 +317,6 @@ class LaunchProject:
                             f"Entrypoint file: {program_name} does not exist, "
                             "and could not be downloaded. Please specify the entrypoint for this run."
                         )
-                    # if the entrypoint is downloaded and inserted into the project dir
-                    # need to rebuild image with new code
-                    self.build_image = True
 
             if (
                 "_session_history.ipynb" in os.listdir(self.project_dir)
@@ -273,8 +335,6 @@ class LaunchProject:
                 self.project_dir,
             )
 
-            # Specify the python runtime for jupyter2docker
-            self.python_version = run_info.get("python", "3")
             if not self._entry_points:
                 _, ext = os.path.splitext(program_name)
                 if ext == ".py":
@@ -362,6 +422,7 @@ def create_project_from_spec(launch_spec: Dict[str, Any], api: Api) -> LaunchPro
         name = launch_spec["name"]
     return LaunchProject(
         launch_spec.get("uri"),
+        launch_spec.get("job"),
         api,
         launch_spec,
         launch_spec["entity"],
@@ -395,9 +456,11 @@ def fetch_and_validate_project(
     if launch_project.source == LaunchSource.LOCAL:
         if not launch_project._entry_points:
             wandb.termlog(
-                "Entry point for local directory not specified, defaulting to `python main.py`"
+                "Entry point for repo not specified, defaulting to `python main.py`"
             )
             launch_project.add_entry_point(["python", "main.py"])
+    elif launch_project.source == LaunchSource.JOB:
+        launch_project._fetch_job()
     else:
         launch_project._fetch_project_local(internal_api=api)
 
