@@ -17,6 +17,7 @@ from .utils import (
     PROJECT_DOCKER_ARGS,
     PROJECT_SYNCHRONOUS,
     resolve_build_and_registry_config,
+    validate_launch_spec_source,
 )
 
 _logger = logging.getLogger(__name__)
@@ -37,6 +38,7 @@ def resolve_agent_config(
         "base_url": api.settings("base_url"),
         "registry": {},
         "build": {},
+        "runner": {},
     }
 
     resolved_config: Dict[str, Any] = defaults
@@ -95,6 +97,7 @@ def create_and_run_agent(
 
 def _run(
     uri: Optional[str],
+    job: Optional[str],
     name: Optional[str],
     project: Optional[str],
     entity: Optional[str],
@@ -113,6 +116,7 @@ def _run(
     """Helper that delegates to the project-running method corresponding to the passed-in backend."""
     launch_spec = construct_launch_spec(
         uri,
+        job,
         api,
         name,
         project,
@@ -127,6 +131,7 @@ def _run(
         cuda,
         run_id,
     )
+    validate_launch_spec_source(launch_spec)
     launch_project = create_project_from_spec(launch_spec, api)
     launch_project = fetch_and_validate_project(launch_project, api)
 
@@ -169,8 +174,9 @@ def _run(
 
 
 def run(
-    uri: Optional[str],
     api: Api,
+    uri: Optional[str] = None,
+    job: Optional[str] = None,
     entry_point: Optional[List[str]] = None,
     version: Optional[str] = None,
     parameters: Optional[Dict[str, Any]] = None,
@@ -189,6 +195,7 @@ def run(
 
     Arguments:
     uri: URI of experiment to run. A wandb run uri or a Git repository URI.
+    job: string reference to a wandb.Job eg: wandb/test/my-job:latest
     api: An instance of a wandb Api from wandb.apis.internal.
     entry_point: Entry point to run within the project. Defaults to using the entry point used
         in the original run for wandb URIs, or main.py for git repository URIs.
@@ -237,6 +244,7 @@ def run(
 
     submitted_run_obj = _run(
         uri=uri,
+        job=job,
         name=name,
         project=project,
         entity=entity,
