@@ -2,19 +2,22 @@
 apikey util.
 """
 
-import getpass
+from functools import partial
 import os
 import stat
 import sys
 import textwrap
 from urllib.parse import urlparse
 
+import click
 import requests
 import wandb
 from wandb.apis import InternalApi
 from wandb.errors import term
 from wandb.util import _is_databricks, isatty, prompt_choices
 
+from .wburls import wburls
+
 
 LOGIN_CHOICE_ANON = "Private W&B dashboard, no account required"
 LOGIN_CHOICE_NEW = "Create a W&B account"
@@ -29,6 +32,9 @@ LOGIN_CHOICES = [
 ]
 
 
+getpass = partial(click.prompt, hide_input=True, err=True)
+
+
 def _fixup_anon_mode(default):
     # Convert weird anonymode values from legacy settings files
     # into one of our expected values.
@@ -53,7 +59,7 @@ def prompt_api_key(  # noqa: C901
         None - if dryrun is selected
         False - if unconfigured (notty)
     """
-    input_callback = input_callback or getpass.getpass
+    input_callback = input_callback or getpass
     log_string = term.LOG_STRING
     api = api or InternalApi(settings)
     anon_mode = _fixup_anon_mode(settings.anonymous)
@@ -94,7 +100,7 @@ def prompt_api_key(  # noqa: C901
 
     api_ask = (
         f"{log_string}: Paste an API key from your profile and hit enter, "
-        "or press ctrl+c to quit: "
+        "or press ctrl+c to quit"
     )
     if result == LOGIN_CHOICE_ANON:
         key = api.create_anonymous_api_key()
@@ -114,6 +120,14 @@ def prompt_api_key(  # noqa: C901
         key = browser_callback() if browser_callback else None
 
         if not key:
+            if not (settings.is_local or local):
+                host = app_url
+                for prefix in "http://", "https://":
+                    if app_url.startswith(prefix):
+                        host = app_url[len(prefix) :]
+                wandb.termlog(
+                    f"Logging into {host}. (Learn how to deploy a W&B server locally: {wburls.get('wandb_server')})"
+                )
             wandb.termlog(
                 f"You can find your API key in your browser here: {app_url}/authorize"
             )
