@@ -11,28 +11,20 @@ class PreInitObject:
             self.__doc__ = destination.__doc__
 
     def __getitem__(self, key: str) -> None:
-        raise wandb.Error(
-            'You must call wandb.init() before {}["{}"]'.format(self._name, key)
-        )
+        raise wandb.Error(f'You must call wandb.init() before {self._name}["{key}"]')
 
     def __setitem__(self, key: str, value: Any) -> Any:
-        raise wandb.Error(
-            'You must call wandb.init() before {}["{}"]'.format(self._name, key)
-        )
+        raise wandb.Error(f'You must call wandb.init() before {self._name}["{key}"]')
 
     def __setattr__(self, key: str, value: Any) -> Any:
         if not key.startswith("_"):
-            raise wandb.Error(
-                "You must call wandb.init() before {}.{}".format(self._name, key)
-            )
+            raise wandb.Error(f"You must call wandb.init() before {self._name}.{key}")
         else:
             return object.__setattr__(self, key, value)
 
     def __getattr__(self, key: str) -> Any:
         if not key.startswith("_"):
-            raise wandb.Error(
-                "You must call wandb.init() before {}.{}".format(self._name, key)
-            )
+            raise wandb.Error(f"You must call wandb.init() before {self._name}.{key}")
         else:
             raise AttributeError()
 
@@ -41,7 +33,7 @@ def PreInitCallable(  # noqa: N802
     name: str, destination: Optional[Any] = None
 ) -> Callable:
     def preinit_wrapper(*args: Any, **kwargs: Any) -> Any:
-        raise wandb.Error("You must call wandb.init() before {}()".format(name))
+        raise wandb.Error(f"You must call wandb.init() before {name}()")
 
     preinit_wrapper.__name__ = str(name)
     if destination:
