@@ -8,8 +8,8 @@ from typing import Any, Callable, Generic, Optional, Tuple, Type, TypeVar
 
 from requests import HTTPError
 import wandb
+from wandb.util import CheckRetryFnType
 
-ExceptionPredicate = Callable[[Exception], bool]
 
 logger = logging.getLogger(__name__)
 
@@ -45,11 +45,14 @@ class Retry(Generic[_R]):
         call_fn: Callable[..., _R],
         retry_timedelta: Optional[datetime.timedelta] = None,
         num_retries: Optional[int] = None,
-        check_retry_fn: ExceptionPredicate = lambda e: True,
+        check_retry_fn: CheckRetryFnType = lambda e: True,
         retryable_exceptions: Optional[Tuple[Type[Exception], ...]] = None,
         error_prefix: str = "Network error",
         retry_callback: Optional[Callable[[int, str], Any]] = None,
         sleep_fn_for_testing: Callable[[float], None] = time.sleep,
+        datetime_now_fn_for_testing: Callable[
+            [], datetime.datetime
+        ] = datetime.datetime.now,
     ) -> None:
         self._call_fn = call_fn
         self._check_retry_fn = check_retry_fn
@@ -64,6 +67,7 @@ class Retry(Generic[_R]):
         self._index = 0
         self.retry_callback = retry_callback
         self._sleep_fn = sleep_fn_for_testing
+        self._datetime_now_fn = datetime_now_fn_for_testing
 
     @property
     def num_iters(self) -> int:
@@ -93,13 +97,14 @@ class Retry(Generic[_R]):
         sleep_base: float = kwargs.pop("retry_sleep_base", 1)
 
         # an extra function to allow performing more logic on the filtered exception
-        check_retry_fn: ExceptionPredicate = kwargs.pop(
+        check_retry_fn: CheckRetryFnType = kwargs.pop(
             "check_retry_fn", self._check_retry_fn
         )
 
         sleep = sleep_base
-        start_time = datetime.datetime.now()
-        now = start_time
+        now = self._datetime_now_fn()
+        start_time = now
+        start_time_triggered = None
 
         self._num_iter = 0
 
@@ -110,24 +115,41 @@ class Retry(Generic[_R]):
                 if self._num_iter > 2 and now - self._last_print > datetime.timedelta(
                     minutes=1
                 ):
-                    self._last_print = datetime.datetime.now()
+                    self._last_print = self._datetime_now_fn()
                     if self.retry_callback:
                         self.retry_callback(
                             200,
                             "{} resolved after {}, resuming normal operation.".format(
-                                self._error_prefix, datetime.datetime.now() - start_time
+                                self._error_prefix, self._datetime_now_fn() - start_time
                             ),
                         )
                 return result
             except self._retryable_exceptions as e:
                 # if the secondary check fails, re-raise
-                if not check_retry_fn(e):
+                retry_timedelta_triggered = check_retry_fn(e)
+                if not retry_timedelta_triggered:
                     raise
-                if (
-                    datetime.datetime.now() - start_time >= retry_timedelta
-                    or self._num_iter >= num_retries
-                ):
+
+                # always enforce num_retries no matter which type of exception was seen
+                if self._num_iter >= num_retries:
                     raise
+
+                now = self._datetime_now_fn()
+
+                # handle a triggered secondary check which could have a shortened timeout
+                if isinstance(retry_timedelta_triggered, datetime.timedelta):
+                    # save the time of the first secondary trigger
+                    if not start_time_triggered:
+                        start_time_triggered = now
+
+                    # make sure that we havent run out of time from secondary trigger
+                    if now - start_time_triggered >= retry_timedelta_triggered:
+                        raise
+
+                # always enforce the default timeout from start of retries
+                if now - start_time >= retry_timedelta:
+                    raise
+
                 if self._num_iter == 2:
                     logger.exception("Retry attempt failed:")
                     if (
@@ -151,7 +173,7 @@ class Retry(Generic[_R]):
             sleep *= 2
             if sleep > self.MAX_SLEEP_SECONDS:
                 sleep = self.MAX_SLEEP_SECONDS
-            now = datetime.datetime.now()
+            now = self._datetime_now_fn()
 
             self._num_iter += 1
 
