@@ -95,6 +95,11 @@ def sweep(
     if callable(sweep):
         sweep = sweep()
     """Sweep create for controller api and jupyter (eventually for cli)."""
+
+    # Project may be only found in the sweep config.
+    if project is None and isinstance(sweep, dict):
+        project = sweep.get("project", None)
+
     if entity:
         env.set_entity(entity)
     if project:
