@@ -2,18 +2,16 @@
 sync.
 """
 
-from __future__ import print_function
-
 import datetime
 import fnmatch
 import os
+import queue
 import sys
 import tempfile
 import threading
 import time
+from urllib.parse import quote as url_quote
 
-from six.moves import queue
-from six.moves.urllib.parse import quote as url_quote
 import wandb
 from wandb.proto import wandb_internal_pb2  # type: ignore
 from wandb.sdk.interface.interface_queue import InterfaceQueue
@@ -29,7 +27,7 @@ TFEVENT_SUBSTRING = ".tfevents."
 TMPDIR = tempfile.TemporaryDirectory()
 
 
-class _LocalRun(object):
+class _LocalRun:
     def __init__(self, path, synced=None):
         self.path = path
         self.synced = synced
@@ -54,6 +52,7 @@ class SyncThread(threading.Thread):
         mark_synced=None,
         app_url=None,
         sync_tensorboard=None,
+        log_path=None,
     ):
         threading.Thread.__init__(self)
         # mark this process as internal
@@ -67,6 +66,7 @@ class SyncThread(threading.Thread):
         self._mark_synced = mark_synced
         self._app_url = app_url
         self._sync_tensorboard = sync_tensorboard
+        self._log_path = log_path
 
     def _parse_pb(self, data, exit_pb=None):
         pb = wandb_internal_pb2.Record()
@@ -126,7 +126,7 @@ class SyncThread(threading.Thread):
             if tb_event_files > 0 and sync_item.endswith(WANDB_SUFFIX):
                 wandb.termwarn("Found .wandb file, not streaming tensorboard metrics.")
             else:
-                print("Found {} tfevent files in {}".format(tb_event_files, tb_root))
+                print(f"Found {tb_event_files} tfevent files in {tb_root}")
                 if len(tb_logdirs) > 3:
                     wandb.termwarn(
                         "Found {} directories containing tfevent files. "
@@ -222,6 +222,8 @@ class SyncThread(threading.Thread):
                 raise e
 
     def run(self):
+        if self._log_path is not None:
+            print(f"Find logs at: {self._log_path}")
         for sync_item in self._sync_list:
             tb_event_files, tb_logdirs, tb_root = self._find_tfevent_files(sync_item)
             if os.path.isdir(sync_item):
@@ -230,7 +232,7 @@ class SyncThread(threading.Thread):
                 if tb_root is None and (
                     check_and_warn_old(files) or len(filtered_files) != 1
                 ):
-                    print("Skipping directory: {}".format(sync_item))
+                    print(f"Skipping directory: {sync_item}")
                     continue
                 if len(filtered_files) > 0:
                     sync_item = os.path.join(sync_item, filtered_files[0])
@@ -248,7 +250,7 @@ class SyncThread(threading.Thread):
             try:
                 ds.open_for_scan(sync_item)
             except AssertionError as e:
-                print(".wandb file is empty ({}), skipping: {}".format(e, sync_item))
+                print(f".wandb file is empty ({e}), skipping: {sync_item}")
                 continue
 
             # save exit for final send
@@ -282,13 +284,13 @@ class SyncThread(threading.Thread):
                             url_quote(r.project),
                             url_quote(r.run_id),
                         )
-                        print("Syncing: %s ..." % url, end="")
+                        print("Syncing: %s ... " % url, end="")
                         sys.stdout.flush()
                         shown = True
             sm.finish()
             # Only mark synced if the run actually finished
             if self._mark_synced and not self._view and finished:
-                synced_file = "{}{}".format(sync_item, SYNCED_SUFFIX)
+                synced_file = f"{sync_item}{SYNCED_SUFFIX}"
                 with open(synced_file, "w"):
                     pass
             print("done.")
@@ -305,6 +307,7 @@ class SyncManager:
         view=None,
         verbose=None,
         sync_tensorboard=None,
+        log_path=None,
     ):
         self._sync_list = []
         self._thread = None
@@ -316,6 +319,7 @@ class SyncManager:
         self._view = view
         self._verbose = verbose
         self._sync_tensorboard = sync_tensorboard
+        self._log_path = log_path
 
     def status(self):
         pass
@@ -335,6 +339,7 @@ class SyncManager:
             mark_synced=self._mark_synced,
             app_url=self._app_url,
             sync_tensorboard=self._sync_tensorboard,
+            log_path=self._log_path,
         )
         self._thread.start()
 
@@ -363,9 +368,9 @@ def get_runs(
     all_dirs = os.listdir(base)
     dirs = []
     if include_offline:
-        dirs += filter(lambda d: d.startswith("offline-run-"), all_dirs)
+        dirs += filter(lambda _d: _d.startswith("offline-run-"), all_dirs)
     if include_online:
-        dirs += filter(lambda d: d.startswith("run-"), all_dirs)
+        dirs += filter(lambda _d: _d.startswith("run-"), all_dirs)
     # find run file in each dir
     fnames = []
     dirs.sort()
@@ -388,9 +393,9 @@ def get_runs(
     for f in fnames:
         dname = os.path.dirname(f)
         # TODO(frz): online runs are assumed to be synced, verify from binary log.
-        if os.path.exists("{}{}".format(f, SYNCED_SUFFIX)) or os.path.basename(
-            dname
-        ).startswith("run-"):
+        if os.path.exists(f"{f}{SYNCED_SUFFIX}") or os.path.basename(dname).startswith(
+            "run-"
+        ):
             if include_synced:
                 filtered.append(_LocalRun(dname, True))
         else:
