@@ -3,7 +3,7 @@ import binascii
 import codecs
 import colorsys
 import contextlib
-from datetime import date, datetime
+from datetime import date, datetime, timedelta
 import errno
 import functools
 import gzip
@@ -56,6 +56,8 @@ from wandb.env import error_reporting_enabled, get_app_url, SENTRY_DSN
 from wandb.errors import CommError, term, UsageError
 import yaml
 
+CheckRetryFnType = Callable[[Exception], Union[bool, timedelta]]
+
 logger = logging.getLogger(__name__)
 _not_importable = set()
 
@@ -887,16 +889,64 @@ def no_retry_auth(e: Any) -> bool:
         raise CommError("Permission denied, ask the project owner to grant you access")
 
 
-def check_retry_commit_artifact(e: Any) -> bool:
+def check_retry_conflict(e: Any) -> Optional[bool]:
+    """Check if the exception is a conflict type so it can be retried.
+
+    Returns:
+        True - Should retry this operation
+        False - Should not retry this operation
+        None - No decision, let someone else decide
+    """
     if hasattr(e, "exception"):
         e = e.exception
-    if (
-        isinstance(e, requests.HTTPError)
-        and e.response is not None
-        and e.response.status_code == 409
-    ):
+    if isinstance(e, requests.HTTPError) and e.response is not None:
+        if e.response.status_code == 409:
+            return True
+    return None
+
+
+def check_retry_conflict_or_gone(e: Any) -> Optional[bool]:
+    """Check if the exception is a conflict or gone type so it can be retried or not.
+
+    Returns:
+        True - Should retry this operation
+        False - Should not retry this operation
+        None - No decision, let someone else decide
+    """
+    if hasattr(e, "exception"):
+        e = e.exception
+    if isinstance(e, requests.HTTPError) and e.response is not None:
+        if e.response.status_code == 409:
+            return True
+        if e.response.status_code == 410:
+            return False
+    return None
+
+
+def make_check_retry_fn(
+    fallback_retry_fn: CheckRetryFnType,
+    check_fn: Callable[[Exception], Optional[bool]],
+    check_timedelta: Optional[timedelta] = None,
+) -> CheckRetryFnType:
+    """Return a check_retry_fn which can be used by lib.Retry().
+
+    Arguments:
+        fallback_fn: Use this function if check_fn didn't decide if a retry should happen.
+        check_fn: Function which returns bool if retry should happen or None if unsure.
+        check_timedelta: Optional retry timeout if we check_fn matches the exception
+    """
+
+    def check_retry_fn(e: Exception) -> Union[bool, timedelta]:
+        check = check_fn(e)
+        if check is None:
+            return fallback_retry_fn(e)
+        if check is False:
+            return False
+        if check_timedelta:
+            return check_timedelta
         return True
-    return no_retry_auth(e)
+
+    return check_retry_fn
 
 
 def find_runner(program: str) -> Union[None, list, List[str]]:
@@ -1102,7 +1152,9 @@ def async_call(target: Callable, timeout: Optional[int] = None) -> Callable:
     return wrapper
 
 
-def read_many_from_queue(q: "queue.Queue", max_items: int, queue_timeout: int) -> list:
+def read_many_from_queue(
+    q: "queue.Queue", max_items: int, queue_timeout: Union[int, float]
+) -> list:
     try:
         item = q.get(True, queue_timeout)
     except queue.Empty:
@@ -1647,7 +1699,7 @@ def _resolve_aliases(aliases: Optional[Union[str, List[str]]]) -> List[str]:
     return aliases
 
 
-def _is_artifact(v: Any) -> bool:
+def _is_artifact_object(v: Any) -> bool:
     return isinstance(v, wandb.Artifact) or isinstance(v, wandb.apis.public.Artifact)
 
 
@@ -1655,7 +1707,19 @@ def _is_artifact_string(v: Any) -> bool:
     return isinstance(v, str) and v.startswith("wandb-artifact://")
 
 
-def parse_artifact_string(v: str) -> Tuple[str, Optional[str]]:
+def _is_artifact_version_weave_dict(v: Any) -> bool:
+    return isinstance(v, dict) and v.get("_type") == "artifactVersion"
+
+
+def _is_artifact_representation(v: Any) -> bool:
+    return (
+        _is_artifact_object(v)
+        or _is_artifact_string(v)
+        or _is_artifact_version_weave_dict(v)
+    )
+
+
+def parse_artifact_string(v: str) -> Tuple[str, Optional[str], bool]:
     if not v.startswith("wandb-artifact://"):
         raise ValueError(f"Invalid artifact string: {v}")
     parsed_v = v[len("wandb-artifact://") :]
@@ -1670,7 +1734,7 @@ def parse_artifact_string(v: str) -> Tuple[str, Optional[str]]:
         # for now can't fetch paths but this will be supported in the future
         # when we allow passing typed media objects, this can be extended
         # to include paths
-        return parts[1], base_uri
+        return parts[1], base_uri, True
 
     if len(parts) < 3:
         raise ValueError(f"Invalid artifact string: {v}")
@@ -1679,7 +1743,7 @@ def parse_artifact_string(v: str) -> Tuple[str, Optional[str]]:
     # when we allow passing typed media objects, this can be extended
     # to include paths
     entity, project, name_and_alias_or_version = parts[:3]
-    return f"{entity}/{project}/{name_and_alias_or_version}", base_uri
+    return f"{entity}/{project}/{name_and_alias_or_version}", base_uri, False
 
 
 def _get_max_cli_version() -> Union[str, None]:
@@ -1703,3 +1767,19 @@ def ensure_text(
         return string
     else:
         raise TypeError(f"not expecting type '{type(string)}'")
+
+
+def make_artifact_name_safe(name: str) -> str:
+    """Make an artifact name safe for use in artifacts"""
+    # artifact names may only contain alphanumeric characters, dashes, underscores, and dots.
+    return re.sub(r"[^a-zA-Z0-9_\-.]", "_", name)
+
+
+def make_docker_image_name_safe(name: str) -> str:
+    """Make a docker image name safe for use in artifacts"""
+    return re.sub(r"[^a-z0-9_\-.]", "", name)
+
+
+def has_main_file(path: str) -> bool:
+    """Check if a directory has a main.py file"""
+    return path != "<python with no main file>"
